@@ -22,6 +22,7 @@ import os
 import unittest
 from pathlib import Path
 import pwd
+import pytest
 from io import BytesIO as StringIO
 
 
@@ -126,6 +127,7 @@ class NetworkExtensionTest(unittest.TestCase):
         # "em.Error: interpreter stdout proxy lost"
         em.Interpreter._wasProxyInstalled = False
 
+    @pytest.mark.docker
     def test_network_extension(self):
         plugins = list_plugins()
         network_plugin = plugins['network']
@@ -144,6 +146,71 @@ class NetworkExtensionTest(unittest.TestCase):
         args = p.get_docker_args(mock_cliargs)
         self.assertTrue('--network host' in args)
 
+class ExposeExtensionTest(unittest.TestCase):
+
+    def setUp(self):
+        # Work around interference between empy Interpreter
+        # stdout proxy and test runner. empy installs a proxy on stdout
+        # to be able to capture the information.
+        # And the test runner creates a new stdout object for each test.
+        # This breaks empy as it assumes that the proxy has persistent
+        # between instances of the Interpreter class
+        # empy will error with the exception
+        # "em.Error: interpreter stdout proxy lost"
+        em.Interpreter._wasProxyInstalled = False
+
+    @pytest.mark.docker
+    def test_expose_extension(self):
+        plugins = list_plugins()
+        expose_plugin = plugins['expose']
+        self.assertEqual(expose_plugin.get_name(), 'expose')
+
+        p = expose_plugin()
+        self.assertTrue(plugin_load_parser_correctly(expose_plugin))
+
+        mock_cliargs = {}
+        self.assertEqual(p.get_snippet(mock_cliargs), '')
+        self.assertEqual(p.get_preamble(mock_cliargs), '')
+        args = p.get_docker_args(mock_cliargs)
+        self.assertNotIn('--expose', args)
+
+        mock_cliargs = {'expose': ['80', '8080']}
+        args = p.get_docker_args(mock_cliargs)
+        self.assertIn('--expose 80', args)
+        self.assertIn('--expose 8080', args)
+
+class PortExtensionTest(unittest.TestCase):
+
+    def setUp(self):
+        # Work around interference between empy Interpreter
+        # stdout proxy and test runner. empy installs a proxy on stdout
+        # to be able to capture the information.
+        # And the test runner creates a new stdout object for each test.
+        # This breaks empy as it assumes that the proxy has persistent
+        # between instances of the Interpreter class
+        # empy will error with the exception
+        # "em.Error: interpreter stdout proxy lost"
+        em.Interpreter._wasProxyInstalled = False
+
+    @pytest.mark.docker
+    def test_port_extension(self):
+        plugins = list_plugins()
+        port_plugin = plugins['port']
+        self.assertEqual(port_plugin.get_name(), 'port')
+
+        p = port_plugin()
+        self.assertTrue(plugin_load_parser_correctly(port_plugin))
+
+        mock_cliargs = {}
+        self.assertEqual(p.get_snippet(mock_cliargs), '')
+        self.assertEqual(p.get_preamble(mock_cliargs), '')
+        args = p.get_docker_args(mock_cliargs)
+        self.assertNotIn('-p', args)
+
+        mock_cliargs = {'port': ['80:8080', '81:8081']}
+        args = p.get_docker_args(mock_cliargs)
+        self.assertIn('-p 80:8080', args)
+        self.assertIn('-p 81:8081', args)
 
 class NameExtensionTest(unittest.TestCase):
 
@@ -250,6 +317,23 @@ class UserExtensionTest(unittest.TestCase):
         self.assertFalse('mkhomedir_helper' in p.get_snippet(home_active_cliargs))
 
         user_override_active_cliargs = mock_cliargs
+        user_override_active_cliargs['user_preserve_groups'] = []
+        snippet_result = p.get_snippet(user_override_active_cliargs)
+        self.assertTrue('usermod -aG' in snippet_result)
+
+        user_override_active_cliargs = mock_cliargs
+        user_override_active_cliargs['user_preserve_groups'] = ['cdrom', 'audio']
+        snippet_result = p.get_snippet(user_override_active_cliargs)
+        self.assertTrue('cdrom' in snippet_result)
+        self.assertTrue('audio' in snippet_result)
+
+        user_override_active_cliargs = mock_cliargs
+        user_override_active_cliargs['user_preserve_groups'] = []
+        user_override_active_cliargs['user_preserve_groups_permissive'] = True
+        snippet_result = p.get_snippet(user_override_active_cliargs)
+        self.assertTrue('usermod -aG' in snippet_result)
+        self.assertTrue('user-preserve-group-permissive Enabled' in snippet_result)
+
         user_override_active_cliargs['user_override_name'] = 'testusername'
         snippet_result = p.get_snippet(user_override_active_cliargs)
         self.assertTrue('USER testusername' in snippet_result)
@@ -260,6 +344,18 @@ class UserExtensionTest(unittest.TestCase):
         snippet_result = p.get_snippet(user_override_active_cliargs)
         self.assertFalse('userdel -r' in snippet_result)
 
+        snippet_result = p.get_snippet(user_override_active_cliargs)
+        self.assertTrue(('-s ' + pwd.getpwuid(os.getuid()).pw_shell) in snippet_result)
+
+        user_override_active_cliargs['user_override_shell'] = 'testshell'
+        snippet_result = p.get_snippet(user_override_active_cliargs)
+        self.assertTrue('-s testshell' in snippet_result)
+
+        user_override_active_cliargs['user_override_shell'] = ''
+        snippet_result = p.get_snippet(user_override_active_cliargs)
+        self.assertFalse('-s' in snippet_result)
+
+    @pytest.mark.docker
     def test_user_collisions(self):
         plugins = list_plugins()
         user_plugin = plugins['user']
@@ -421,3 +517,37 @@ class EnvExtensionTest(unittest.TestCase):
         self.assertEqual(p.get_snippet(mock_cliargs), '')
         self.assertEqual(p.get_preamble(mock_cliargs), '')
         self.assertEqual(p.get_docker_args(mock_cliargs), ' --env-file foo --env-file bar')
+
+
+class GroupAddExtensionTest(unittest.TestCase):
+
+    def setUp(self):
+        # Work around interference between empy Interpreter
+        # stdout proxy and test runner. empy installs a proxy on stdout
+        # to be able to capture the information.
+        # And the test runner creates a new stdout object for each test.
+        # This breaks empy as it assumes that the proxy has persistent
+        # between instances of the Interpreter class
+        # empy will error with the exception
+        # "em.Error: interpreter stdout proxy lost"
+        em.Interpreter._wasProxyInstalled = False
+
+    @pytest.mark.docker
+    def test_group_add_extension(self):
+        plugins = list_plugins()
+        group_add_plugin = plugins['group_add']
+        self.assertEqual(group_add_plugin.get_name(), 'group_add')
+
+        p = group_add_plugin()
+        self.assertTrue(plugin_load_parser_correctly(group_add_plugin))
+
+        mock_cliargs = {}
+        self.assertEqual(p.get_snippet(mock_cliargs), '')
+        self.assertEqual(p.get_preamble(mock_cliargs), '')
+        args = p.get_docker_args(mock_cliargs)
+        self.assertNotIn('--group_add', args)
+
+        mock_cliargs = {'group_add': ['sudo', 'docker']}
+        args = p.get_docker_args(mock_cliargs)
+        self.assertIn('--group-add sudo', args)
+        self.assertIn('--group-add docker', args)
